#ifndef AUBO_SDK_ROBOT_CONTROL_INTERFACE_H
#define AUBO_SDK_ROBOT_CONTROL_INTERFACE_H

#include <vector>
#include <thread>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

class ARCS_ABI_EXPORT RobotManage
{
public:
    RobotManage();
    virtual ~RobotManage();

    /**
     * 发起机器人上电请求
     */
    int poweron();

    /**
     * 发起机器人启动请求
     */
    int startup();

    /**
     * 发起机器人断电请求
     */
    int poweroff();

    /**
     * 发起机器人反向驱动请求
     */
    int backdrive(bool enable);

    /**
     * 发起机器人自由驱动请求
     */
    int freedrive(bool enable);

    /**
     * 发起机器人进入/退出仿真模式请求
     */
    int setSim(bool enable);

    /**
     * 设置机器人操作模式
     */
    int setOperationalMode(OperationalModeType mode);

    /**
     * 获取操作模式
     */
    OperationalModeType getOperationalMode();

    /**
     * 获取控制模式
     */
    RobotControlModeType getRobotControlMode();

    /**
     * 是否使能了拖动示教模式
     */
    bool isFreedriveEnabled();

    /**
     * 是否使能了反向驱动模式
     */
    bool isBackdriveEnabled();

    /**
     * 是否使能了仿真模式
     */
    bool isSimulationEnabled();

    /**
     * setUnlockProtectiveStop
     *
     * @return
     */
    int setUnlockProtectiveStop();

    /**
     * 重置安全接口板
     *
     * @return
     */
    int restartInterfaceBoard();

protected:
    void *d_;
};
using RobotManagePtr = std::shared_ptr<RobotManage>;

// clang-format off
#define ROBOT_MANAGE_DECLARES                    \
    X(RobotManage, 0, poweron)                   \
    X(RobotManage, 0, startup)                   \
    X(RobotManage, 0, poweroff)                  \
    X(RobotManage, 1, backdrive, enable)         \
    X(RobotManage, 1, freedrive, enable)         \
    X(RobotManage, 1, setSim, enable)            \
    X(RobotManage, 1, setOperationalMode, mode)  \
    X(RobotManage, 0, getOperationalMode)        \
    X(RobotManage, 0, getRobotControlMode)       \
    X(RobotManage, 0, isSimulationEnabled)       \
    X(RobotManage, 0, isFreedriveEnabled)        \
    X(RobotManage, 0, isBackdriveEnabled)        \
    X(RobotManage, 0, setUnlockProtectiveStop)   \
    X(RobotManage, 0, restartInterfaceBoard)
// clang-format on
} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_ROBOT_CONTROL_INTERFACE_H
