#ifndef AUBO_SDK_SYSTEM_INFO_INTERFACE_H
#define AUBO_SDK_SYSTEM_INFO_INTERFACE_H

#include <stdint.h>
#include <string>
#include <memory>

#include <aubo/global_config.h>

namespace arcs {
namespace common_interface {

class ARCS_ABI_EXPORT SystemInfo
{
public:
    SystemInfo();
    virtual ~SystemInfo();

    /// 获取控制器软件版本号
    int getControlSoftwareVersionCode();

    /// 获取接口版本号
    int getInterfaceVersionCode();

    /// 获取控制器软件构建时间
    std::string getControlSoftwareBuildDate();

    /// 获取控制器软件git版本
    std::string getControlSoftwareVersionHash();

    /// 获取系统时间(软件启动时间us)
    uint64_t getControlSystemTime();

protected:
    void *d_;
};

using SystemInfoPtr = std::shared_ptr<SystemInfo>;

// clang-format off
#define SYSTEM_INFO_DECLARES                         \
    X(SystemInfo, 0, getControlSoftwareVersionCode)  \
    X(SystemInfo, 0, getInterfaceVersionCode)        \
    X(SystemInfo, 0, getControlSoftwareBuildDate)    \
    X(SystemInfo, 0, getControlSoftwareVersionHash)  \
    X(SystemInfo, 0, getControlSystemTime)
// clang-format on
} // namespace common_interface
} // namespace arcs
#endif
